@@ -21,10 +21,10 @@
 
 #include "dialog_fieldcalculation.h"
 #include "../../box_db_table.h"
-#include "../../frame_glom.h"
-#include "../../python_embed/glom_python.h"
-#include <glom/libglom/data_structure/glomconversions.h>
-#include <gtksourceviewmm/sourcelanguagesmanager.h>
+#include <glom/frame_glom.h>
+#include <glom/python_embed/glom_python.h>
+#include <libglom/data_structure/glomconversions.h>
+#include <gtksourceviewmm/sourcelanguagemanager.h>
 
 //#include <libgnome/gnome-i18n.h>
 #include <glibmm/i18n.h>
@@ -32,12 +32,12 @@
 namespace Glom
 {
 
-Dialog_FieldCalculation::Dialog_FieldCalculation(BaseObjectType* cobject, const Glib::RefPtr<Gnome::Glade::Xml>& refGlade)
+Dialog_FieldCalculation::Dialog_FieldCalculation(BaseObjectType* cobject, const Glib::RefPtr<Gtk::Builder>& builder)
 : Gtk::Dialog(cobject)
 {
-  refGlade->get_widget("textview_calculation",  m_text_view);
-  refGlade->get_widget("button_test",  m_button_test);
-  refGlade->get_widget("label_triggered_by", m_label_triggered_by);
+  builder->get_widget("textview_calculation",  m_text_view);
+  builder->get_widget("button_test",  m_button_test);
+  builder->get_widget("label_triggered_by", m_label_triggered_by);
 
   m_button_test->signal_clicked().connect( sigc::mem_fun(*this, &Dialog_FieldCalculation::on_button_test) );
   //on_foreach_connect(*this);
@@ -46,15 +46,14 @@ Dialog_FieldCalculation::Dialog_FieldCalculation(BaseObjectType* cobject, const
   {
     m_text_view->set_highlight_current_line(true);
 
-    //TODO: Shouldn't this be a singleton?
-    Glib::RefPtr<gtksourceview::SourceLanguagesManager> languages_manager = gtksourceview::SourceLanguagesManager::create();
+    Glib::RefPtr<gtksourceview::SourceLanguageManager> languages_manager = gtksourceview::SourceLanguageManager::get_default();
 
-    Glib::RefPtr<gtksourceview::SourceLanguage> language = languages_manager->get_language_from_mime_type("application/x-python");
+    Glib::RefPtr<gtksourceview::SourceLanguage> language = languages_manager->get_language("python"); //This is the GtkSourceView language ID.
     if(language)
     {
        //Createa a new buffer and set it, instead of getting the default buffer, in case libglade has tried to set it, using the wrong buffer type:
        Glib::RefPtr<gtksourceview::SourceBuffer> buffer = gtksourceview::SourceBuffer::create(language);
-       buffer->set_highlight();
+       buffer->set_highlight_syntax();
        m_text_view->set_buffer(buffer);
     }
   }
@@ -92,17 +91,30 @@ sharedptr<Field> Dialog_FieldCalculation::get_field() const
   return field;
 }
 
+bool Dialog_FieldCalculation::check_for_return_statement(const Glib::ustring& calculation)
+{
+  if(calculation.find("return") == Glib::ustring::npos)
+  {
+     Frame_Glom::show_ok_dialog(_("Calculation Error"), _("The calculation does not have a return statement."), *this);
+     return false;
+  }
+  
+  return true;
+}
+
 void Dialog_FieldCalculation::on_button_test()
 {
   const Glib::ustring calculation = m_text_view->get_buffer()->get_text();
+  if(!check_for_return_statement(calculation))
+    return;
 
   type_map_fields field_values;
 
-  Document_Glom* document = get_document();
+  Document* document = get_document();
   if(document)
   {
-    const Document_Glom::type_vecFields fields = document->get_table_fields(m_table_name);
-    for(Document_Glom::type_vecFields::const_iterator iter = fields.begin(); iter != fields.end(); ++iter)
+    const Document::type_vec_fields fields = document->get_table_fields(m_table_name);
+    for(Document::type_vec_fields::const_iterator iter = fields.begin(); iter != fields.end(); ++iter)
     {
       const sharedptr<const Field> field = *iter;
       const Gnome::Gda::Value example_value = Conversions::get_example_value(field->get_glom_type());
